--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR = "combo"

local Addon  = LibStub( "AceAddon-3.0"):GetAddon( "Bison")
local L      = LibStub( "AceLocale-3.0"):GetLocale( "Bison")
local AceGUI = LibStub( "AceGUI-3.0")

local NOTHING = {}
local UPDATE_TIME = 0.2
local MAX_BUTTON  = 5

local Module = Addon:NewBarModule( MAJOR)
Module.color   = { r=0.2, g=0.4, b=0.9 }
Module.proName = "combo"

local comboTable = NOTHING
local args = {
	option = { 
		type = "group", order = 10, name = L.OptionName, inline = true, 
		args = {
			show     = { type = "toggle", order = 10, name = L.ShowName,     desc = L.ShowDesc, width = "full" },
--			sort     = { type = "select", order = 20, name = L.SortName,     desc = L.SortDesc, get = "GetSortType", set = "SetSortType", values = "GetSortDesc" },
			timer    = { type = "select", order = 30, name = L.TimerName,    desc = L.TimerDesc, values = "GetTimerDesc" },
			flashing = { type = "toggle", order = 40, name = L.FlashingName, desc = L.FlashingDesc },
--			spell    = { type = "toggle", order = 50, name = L.SpellName,    desc = L.SpellDesc },
		} 
	},
	layout = { 
		type = "group", order = 20, name = L.BarName, inline = true, 
		args = {
			horizontal = { type = "toggle", order = 10, name = L.HorizontalName, desc = L.HorizontalDesc, width = "full" },
--			number     = { type = "range",  order = 30, name = L.NumberName,     desc = L.NumberDesc,   set = "SetNumber",   min = 1,    max = MAX_BUTTON, step = 1 },
			scale      = { type = "range",  order = 40, name = L.ScaleName,      desc = L.ScaleDesc,    set = "SetScale",    min = 0.01, max = 2,          step = 0.01, isPercent = true },
			cols       = { type = "range",  order = 50, name = L.ColsName,       desc = L.ColsDesc,     set = "SetCols",     min = 1,    max = MAX_BUTTON, step = 1 },
			xPadding   = { type = "range",  order = 60, name = L.XPaddingName,   desc = L.XPaddingDesc, set = "SetXPadding", min = -20,  max = 20,         step = 1 },
			rows       = { type = "range",  order = 70, name = L.RowsName,       desc = L.RowsDesc,     set = "SetRows",     min = 1,    max = MAX_BUTTON, step = 1 },
			yPadding   = { type = "range",  order = 80, name = L.YPaddingName,   desc = L.YPaddingDesc, set = "SetYPadding", min = -50,  max = 50,         step = 1 },
--			cooldown   = { type = "toggle", order = 15, name = L.CooldownName,   desc = L.CooldownDesc },
		} 
	}
}
local blizzOptions = {
	type = "group", order = 50, name = L.DescProgs, handler = Module, get = "GetProperty", set = "SetProperty", args = args
}
local dialogOptions = {
	type = "group", order = 20, name = L.BarCombo, handler = Module, get = "GetProperty", set = "SetProperty", args = args,
	plugins = {
		p1 = { 
			descr = { type = "description", order = 5, name = L.DescProgs, fontSize = "large" }
		}
	}
}
local comboIcon
local comboSpell

------------------------------------------------------------------------------------
-- Main
------------------------------------------------------------------------------------
local function ScanAura( unit, id, filter)
	local name = GetSpellInfo( id)
	local _, _, _, count = UnitAura( unit, name, nil, filter)
	return count or 0
end

local function GetPoints()
	local _, class = UnitClass( "player")
	if class == "DEATHKNIGHT" then
		return ScanAura( "pet", 91342, "HELPFUL") -- Shadow Infusion
	elseif class == "DRUID" then
		if GetShapeshiftFormID() == BEAR_FORM then
			return ScanAura( "target", 33745, "HARMFUL") -- Lacerate
		end
		return GetComboPoints( "player")
	elseif class == "HUNTER" then
		if IsSpellKnown( 19434) then		-- Aimed Shot
			return ScanAura( "player", 82925, "HELPFUL") -- Ready, Set, Aim...
		elseif IsSpellKnown( 19577) then	-- Intimidation
			return ScanAura( "pet", 19615, "HELPFUL") -- Frenzy Effect
		end
	elseif class == "MAGE" then
	elseif class == "PALADIN" then
		return UnitPower( "player", SPELL_POWER_HOLY_POWER)
	elseif class == "PRIEST" then
		local auraID = IsSpellKnown( 15407) and 77487 or 81661 -- Mind Flay / Shadow Orbs / Evangelism
		return ScanAura( "player", auraID, "HELPFUL") -- Shadow Infusion
	elseif class == "ROGUE" then
		return GetComboPoints( "player")
	elseif class == "SHAMAN" then
		if IsSpellKnown( 51490) then -- Thunderstorm
			local count = ScanAura( "player", 324, "HELPFUL") -- Lightning Shield
			return count > 0 and count - 4 or 0
		else
			return ScanAura( "player", 53817, "HELPFUL") -- Maelstrom Weapon
		end
	elseif class == "WARLOCK" then
		return UnitPower( "player", SPELL_POWER_SOUL_SHARDS)
	elseif class == "WARRIOR" then
	end
	return 0
end

------------------------------------------------------------------------------------
-- Main
------------------------------------------------------------------------------------
function Module:OnModuleInitialize()
	self:RegisterOptions( blizzOptions, L.BarCombo)
	for i = 1,5 do
		tinsert( self.aura, { id = i, name = "__combo__", count = 1 })
	end
end

function Module:OnModuleEnable()
	self:RegisterEvent( "ACTIVE_TALENT_GROUP_CHANGED")
	self:ACTIVE_TALENT_GROUP_CHANGED()
end

function Module:GetOptionTable()
	return dialogOptions
end

function Module:UpdateAnchors( sort)
	Addon:Debug( self, ":UpdateAnchors", self.profile.show)
	if self.profile.show and comboIcon and PlayerFrame.unit == "player" then
		local combos = GetPoints()
		Addon:Debug( self, ":UpdateAnchors", combos)
		for i,a in pairs( self.aura) do
			a.texture = comboIcon
			local buff = self:GetUserBuff( "BuffComboButton", a.id)
			buff:SetScript( "OnEnter", nil)
			self:UpdateLBF( buff)
			local child = self.group.children[i]
			if child and i <= combos then
				buff:Show()
				local icon = _G[buff:GetName().."Icon"]
				if icon then
					icon:SetTexture( a.texture)
				end
				if GameTooltip:IsOwned( buff) and comboSpell then
--					GameTooltip:SetSpell( comboSpell, "spell")
				end
				child:SetBuff( buff)
			elseif child then
				buff:Hide()
				child:SetBuff( nil)
			else
				buff:Hide()
			end
			self:UpdateBuffScale( buff)
		end
	end
end

function Module:UpdateEnchantAnchors()
	self:UpdateAnchors()
end

function Module:MoveTo( offset)
	for i,child in pairs( self.group.children) do
		child:SetBuff( nil)
	end
	for id = 1,#self.aura do
		local buff = self:GetUserBuff( "BuffComboButton", id)
		buff:Hide()
		buff.duration:Hide()
		buff:SetScript( "OnUpdate", nil)
	end
end

------------------------------------------------------------------------------------
-- Event
------------------------------------------------------------------------------------
function Module:UPDATE_SHAPESHIFT_FORM()
	self:ACTIVE_TALENT_GROUP_CHANGED()
end

function Module:ACTIVE_TALENT_GROUP_CHANGED()
	Addon:Debug( self, ":ACTIVE_TALENT_GROUP_CHANGED", index)
	local _, class = UnitClass( "player")
	if class == "DEATHKNIGHT" then
		comboIcon  = "Interface\\ICONS\\Spell_Shadow_Requiem"
		comboSpell = 91342
	elseif class == "DRUID" then
		if GetShapeshiftFormID() == BEAR_FORM then
			comboIcon = "Interface\\ICONS\\Ability_Druid_Lacerate"
			comboSpell = 33745
		else
			comboIcon = "Interface\\ICONS\\Ability_DualWield"
			comboSpell = nil
		end
		self:RegisterEvent( "UPDATE_SHAPESHIFT_FORM")
	elseif class == "HUNTER" then
		if IsSpellKnown( 19434) then		-- Aimed Shot
			comboIcon = "Interface\\ICONS\\Ability_Hunter_MasterMarksman"
			comboSpell = 82925
		elseif IsSpellKnown( 19577) then	-- Intimidation
			comboIcon = "Interface\\ICONS\\INV_Misc_MonsterClaw_03"
			comboSpell = 19615
		end
	elseif class == "MAGE" then
--		comboIcon = "Interface\\ICONS\\"	-- 
	elseif class == "PALADIN" then
		comboIcon = "Interface\\ICONS\\Spell_Holy_HolyBolt"
--		comboSpell = 19615
	elseif class == "PRIEST" then
		if IsSpellKnown( 15407) then -- Mind Flay
			comboIcon = "Interface\\ICONS\\Spell_Priest_ShadowOrbs"
			comboSpell = 95740
		else
			comboIcon = "Interface\\ICONS\\Spell_Holy_DivineIllumination"
			comboSpell = 81661
		end
	elseif class == "ROGUE" then
		comboIcon = "Interface\\ICONS\\Ability_DualWield"
		comboSpell = nil
	elseif class == "SHAMAN" then
		if IsSpellKnown( 51490) then -- Thunderstorm
			comboIcon = "Interface\\ICONS\\Spell_Nature_LightningShield"
			comboSpell = 324
		else
			comboIcon = "Interface\\ICONS\\Spell_Shaman_MaelstromWeapon"
			comboSpell = 53817
		end
	elseif class == "WARLOCK" then
		comboIcon = "Interface\\ICONS\\INV_Misc_Gem_Amethyst_02"
		comboSpell = nil
	elseif class == "WARRIOR" then
--		comboIcon = "Interface\\ICONS\\"	-- 
	end
end
